// Converts the n-Queens problem to HCP. The only input is the number of queens.
//
//
// Written by Michael Haythorpe

import java.io.*;
import java.util.StringTokenizer;

public class Queen2HCP
{
   public static void main(String [] args)
   {
      if(args.length != 2)
      {
         System.out.println("QUEEN2HCP should be run using the following input:");
         System.out.println();
         System.out.println("QUEEN2HCP.exe <number of queens> <output_file.hcp>");
         System.out.println();
         return;
      }
      
      int N = Integer.parseInt(args[0]);
      String outfile = args[1];
      
      int [][]A = new int[12*N*N + 10*N - 2][12*N*N + 10*N - 2];
      
      
      for(int i=1; i<=N; i++)
         A[0][1+(i-1)*3*N] = 1;
         
      for(int i=1; i<=3*N*N; i++)
      {
         A[1+(i-1)*3][2+(i-1)*3] = 1; A[2+(i-1)*3][1+(i-1)*3] = 1;
         A[2+(i-1)*3][3+(i-1)*3] = 1; A[3+(i-1)*3][2+(i-1)*3] = 1;
      }
      
      int count = 0;
      for(int i=1; i<=N; i++)
         for(int j=1; j<=N; j++)
         {
            count++;
            A[3*count][3*N*N - 2 + 3*count] = 1;
            A[3*N*N + 3*count][6*N*N - 2 + 3*count] = 1;
         }
      
      count = 0;
      for(int i=1; i<=N; i++)
         for(int j=1; j<=N; j++)
         {
            count++;
            if(j != N)
               for(int k=1; k<=N; k++)
                  A[6*N*N + 3*count][1 + 3*N*(k-1) + 3*j] = 1;
            else
               A[6*N*N + 3*count][9*N*N + 1] = 1;
         }
         
      for(int i=1; i<=N; i++)
      {
         A[9*N*N + 2*i - 1][9*N*N + 2*i] = 1;
         A[9*N*N + 2*i][3 + 3*N*(i-1)] = 1;
         A[9*N*N + 10*N - 3 + i*N][9*N*N + 2*i + 1] = 1;
         A[9*N*N + 2*i - 1][3 + 3*N*(i-1)] = 1;
         for(int j=1; j<=N-1; j++)
         {
            A[-2 + 3*N*(i-1) + 3*j][9*N*N + 10*N - 3 + (i-1)*N + j] = 1;
            A[9*N*N + 10*N - 3 + (i-1)*N + j][3 + 3*N*(i-1) + 3*j] = 1;
            A[9*N*N + 10*N - 3 + (i-1)*N + j][9*N*N + 2*i] = 1;
            A[9*N*N + 2*i][9*N*N + 10*N - 3 + (i-1)*N + j] = 1;
         }
         A[9*N*N + 2*i][9*N*N + 10*N - 3 + i*N] = 1;
         A[-2 + 3*N*i][9*N*N + 10*N - 3 + i*N] = 1;
         A[9*N*N + 10*N - 3 + i*N][9*N*N + 2*i + 1] = 1;
      }
      
      int startRow = 1;
      int startCol = N;
      
      for(int i=1; i<=2*N-1; i++)
      {
         A[9*N*N + 2*N + 2*i - 1][9*N*N + 2*N + 2*i] = 1;
         A[9*N*N + 2*N + 2*i][3*N*N + 3*N*(startRow-1) + 3*startCol] = 1;
         A[9*N*N + 10*N - 3 + N*N + (startRow-1)*N + startCol][9*N*N + 2*N + 2*i + 1] = 1;
         A[9*N*N + 2*N + 2*i - 1][3*N*N + 3*N*(startRow-1) + 3*startCol] = 1;
      
         int row = startRow;
         int col = startCol;
         while(row < N && col < N)
         {
            A[3*N*N - 2 + 3*N*(row-1) + 3*col][9*N*N + 10*N - 3 + N*N + (row-1)*N + col] = 1;
            A[9*N*N + 10*N - 3 + N*N + (row-1)*N + col][3*N*N + 3*N*row + 3*(col+1)] = 1;
            A[9*N*N + 10*N - 3 + N*N + (row-1)*N + col][9*N*N + 2*N + 2*i] = 1;
            A[9*N*N + 2*N + 2*i][9*N*N + 10*N - 3 + N*N + (row-1)*N + col] = 1;
            row++;
            col++;
         }
         
         A[9*N*N + 2*N + 2*i][9*N*N + 10*N - 3 + N*N + (row-1)*N + col] = 1;
         A[3*N*N - 2 + 3*N*(row-1) + 3*col][9*N*N + 10*N - 3 + N*N + (row-1)*N + col] = 1;
         A[9*N*N + 10*N - 3 + N*N + (row-1)*N + col][9*N*N + 2*N + 2*i + 1] = 1;
         if(startCol > 1)
            startCol--;
         else
            startRow++;
      }
      
      startRow = 1;
      startCol = 1;
      for(int i=1; i<=2*N-1; i++)
      {
         A[9*N*N + 6*N - 3 + 2*i][9*N*N + 6*N - 2 + 2*i] = 1;
         A[9*N*N + 6*N - 2 + 2*i][6*N*N + 3*N*(startRow-1) + 3*startCol] = 1;
         A[9*N*N + 10*N - 3 + 2*N*N + (startRow-1)*N + startCol][9*N*N + 6*N - 1 + 2*i] = 1;
         A[9*N*N + 6*N - 3 + 2*i][6*N*N + 3*N*(startRow-1) + 3*startCol] = 1;
      
         int row = startRow;
         int col = startCol;
         while(row < N && col > 1)
         {
            A[6*N*N - 2 + 3*N*(row-1) + 3*col][9*N*N + 10*N - 3 + 2*N*N + (row-1)*N + col] = 1;
            A[9*N*N + 10*N - 3 + 2*N*N + (row-1)*N + col][6*N*N + 3*N*row + 3*(col-1)] = 1;
            A[9*N*N + 10*N - 3 + 2*N*N + (row-1)*N + col][9*N*N + 6*N - 2 + 2*i] = 1;
            A[9*N*N + 6*N - 2 + 2*i][9*N*N + 10*N - 3 + 2*N*N + (row-1)*N + col] = 1;
            row++;
            col--;
         }  
         
         A[9*N*N + 6*N - 2 + 2*i][9*N*N + 10*N - 3 + 2*N*N + (row-1)*N + col] = 1;
         A[6*N*N - 2 + 3*N*(row-1) + 3*col][9*N*N + 10*N - 3 + 2*N*N + (row-1)*N + col] = 1;
         A[9*N*N + 10*N - 3 + 2*N*N + (row-1)*N + col][9*N*N + 6*N - 1 + 2*i] = 1;
         
         if(startCol < N)
            startCol++;
         else
            startRow++;
      
      }
      
      A[9*N*N + 10*N - 3][0] = 1;
      
      try
      {
         BufferedWriter bw = new BufferedWriter(new FileWriter(outfile));
         
         bw.write("NAME : Queens Problem for N = " + N);
         bw.newLine();
         bw.write("COMMENT : ");
         bw.newLine();
         bw.write("TYPE : HCP");
         bw.newLine();
         bw.write("DIMENSION : " + (36*N*N + 30*N - 6));
         bw.newLine();
         bw.write("EDGE_DATA_FORMAT : EDGE_LIST");
         bw.newLine();
         bw.write("EDGE_DATA_SECTION");
         bw.newLine();
         
         for(int i=0; i<12*N*N + 10*N - 2; i++)
         {
            bw.write((3*i+1) + " " + (3*i+2));
            bw.newLine();
            for(int j=i+1; j<12*N*N + 10*N - 2; j++)
            {
               if(A[j][i] == 1)
               {
                  bw.write((3*i+1) + " " + (3*j+3));
                  bw.newLine();
               }
            }
            
            bw.write((3*i+2) + " " + (3*i + 3));
            bw.newLine();
            
            for(int j=i+1; j<12*N*N + 10*N - 2; j++)
            {
               if(A[i][j] == 1)
               {
                  bw.write((3*i+3) + " " + (3*j+1));
                  bw.newLine();
               }
            }
         }
         
         bw.write("-1");
         bw.newLine();
         bw.write("EOF");
         bw.close();
      
      }
      catch(Exception e)
      {
         System.out.println(e);
      }
      
   }
}